// Description: a mover that takes end coordinates from a node
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> the node message can also indicate a user head, wand, or body
//</ul>
//
// Category: Transformation
// Author: Alex Hill
// Revision: 11/01/01
//
#include <ygNodeDB.h>
#include <ygUser.h>
#include <ygNetMessages.h>
#include "moveToNode.h"

#define USER_NONE 100
#define USER_HEAD 101
#define USER_BODY 102
#define USER_WAND 103

extern "C" ygNode* construct_moveToNode(const char* name,bool master) { return new moveToNode(name,master); }

moveToNode::moveToNode(const char* name,bool master) : mover(name,master)
    {
    setClassName("moveToNode");
    node = NULL;
    mode = USER_NONE;
    wandNum = 0;
    addNetMessage("node",&nodeName,YG_NET_STRING);
    addNetMessage("node",&mode,YG_NET_INT);
    addNetMessageMap("node","none",USER_NONE);
    addNetMessageMap("node","head",USER_HEAD);
    addNetMessageMap("node","body",USER_BODY);
    addNetMessageMap("node","wand",USER_WAND);
    addNetMessage("node",&wandNum,YG_NET_INT);
    }

moveToNode::~moveToNode(void)
    {
    }

void moveToNode::reset(void)
    {
    //clear node name
    node = NULL;
    nodeName.clear();
    mode = USER_NONE;
    wandNum = 0;
    mover::reset();
    }

void moveToNode::message(const ygMessage& msg)
    {
    //clear or set the node and optional user body part
    if (msg == "node")
        {
        //if number of arguments is greater than zero
        if (nodeName.length() > 0)
            {
            node = ygNodeDB::find(nodeName);
            //if node type is user then
            if (node && node->isOfClass("ygUser"))
                {
                ygUser* user = (ygUser*)node;
                //if second argument is "head"
                if (mode == USER_HEAD)
                    {
                    //set node to user head
                    if (user->head())
                        node = user->head();
                    }
                //if second argument is "body"
                else if (mode == USER_BODY)
                    {
                    //for each user child
                    for (int i=0; i<user->numChildren(); i++)
                        {
                        ygNode* currNode = user->child(i);
                        //if child is of type body then set node
                        if (currNode->isOfClass("body"))
                            node = currNode;
                        }
                    }
                //if second argument is "wand"
                else if (mode == USER_WAND)
                    {
                    //if user has a wand then
                    if (user->wand(wandNum))
                        {
                        //use specified wand number
                        node = user->wand(wandNum);
                        }
                    }
                else
                    node = user;
                }
            }
        //else, clear the node name
        else
            node = NULL;
        }
    else
        mover::message(msg);
    }


void moveToNode::app(void)
    {
    if (node)
        {
        endPosQuat.pos = node->origin(parent());
        pfMatrix mat;
        node->getTransform(mat);
        mat.getOrthoQuat(endPosQuat.quat);
        }
    mover::app();
    }
