// Author: Dave Pape
//           11/01/01
// Revision: 06/28/05 Alex Hill - changed distance calc to go to -1.0 at center
//
#include "ygBox.h"

ygBox::ygBox(void)
    {
    pfBox::makeEmpty();
    }

ygBox::ygBox(const pfBox & box)
    {
    pfBox::min = box.min;
    pfBox::max = box.max;
    }


ygBox& ygBox::operator=(const pfBox & box)
    {
    pfBox::min = box.min;
    pfBox::max = box.max;
    return *this;
    }


float ygBox::distance(const pfVec3 & v) const
    {
    float dist;
    float dx,dy,dz;
    if (v[0] >= min[0])
        if (v[0] <= max[0])
            dx = 0;
        else
            dx = v[0] - max[0];
    else
        dx = min[0] - v[0];
    if (v[1] >= min[1])
        if (v[1] <= max[1])
            dy = 0;
        else
            dy = v[1] - max[1];
    else
        dy = min[1] - v[1];
    if (v[2] >= min[2])
        if (v[2] <= max[2])
            dz = 0;
        else
            dz = v[2] - max[2];
    else
        dz = min[2] - v[2];
    if (dx == 0 && dy == 0 && dz == 0)
        {
        pfVec3 center;
        center.set((max[0]+min[0])/2,(max[1]+min[1])/2,(max[2]+min[2])/2);
        pfVec3 size;
        size.set((max[0]-min[0])/2,(max[1]-min[1])/2,(max[2]-min[2])/2);
        dx = (v[0] - center[0])/size[0];
        dy = (v[1] - center[1])/size[1];
        dz = (v[2] - center[2])/size[2];
        dist = dx*dx + dy*dy + dz*dz - 1.0;
        if (dist > 0)
            dist = 0;
        }
    else
        dist = sqrtf(dx*dx + dy*dy + dz*dz);
    return dist;
    }


float ygBox::sqrDistance(const pfVec3 & v) const
    {
    float dx,dy,dz;
    if (v[0] >= min[0])
        if (v[0] <= max[0])
            dx = 0;
        else
            dx = v[0] - max[0];
    else
        dx = min[0] - v[0];
    if (v[1] >= min[1])
        if (v[1] <= max[1])
            dy = 0;
        else
            dy = v[1] - max[1];
    else
        dy = min[1] - v[1];
    if (v[2] >= min[2])
        if (v[2] <= max[2])
            dz = 0;
        else
            dz = v[2] - max[2];
    else
        dz = min[2] - v[2];
    return dx*dx + dy*dy + dz*dz;
    }


int ygBox::contains(const pfVec3 &v) const
    {
    return pfBox::contains(v);
    }

int ygBox::isect(const pfSeg *s,float *d0,float *d1) const
    {
    return pfBox::isect(s,d0,d1);
    }

void ygBox::makeEmpty(void)
    {
    pfBox::makeEmpty();
    }

void ygBox::extendBy(const pfVec3 & v)
    {
    pfBox::extendBy(v);
    }

void ygBox::extendBy(const pfBox * box)
    {
    pfBox::extendBy(box);
    }

void ygBox::extendBy(const pfSphere * sph)
    {
    pfBox tmpbox;
    tmpbox.around(&sph,1);
    pfBox::extendBy(&tmpbox);
    }

void ygBox::extendBy(const pfCylinder * cyl)
    {
    pfBox tmpbox;
    tmpbox.around(&cyl,1);
    pfBox::extendBy(&tmpbox);
    }

void ygBox::around(const pfVec3 * vert,int nvert)
    {
    pfBox::around(vert,nvert);
    }

void ygBox::around(const pfBox ** boxes,int nboxes)
    {
    pfBox::around(boxes,nboxes);
    }

void ygBox::around(const pfSphere ** sph,int nsph)
    {
    pfBox::around(sph,nsph);
    }

void ygBox::around(const pfCylinder ** cyl,int ncyl)
    {
    pfBox::around(cyl,ncyl);
    }


float ygBox::size(void) const
    {
    pfVec3 delta = max - min;
    return delta.length()/2;
    }


int ygBox::volumeType(void) const
    {
    return YGVOL_BOX;
    }
