#ifndef _ygNode_h_
#define _ygNode_h_

#include "ygString.h"
#include "ygMessage.h"

#define YG_CHANGE_NONE      (1 << 0)
#define YG_CHANGE_STATUS    (1 << 1)
#define YG_CHANGE_TREE      (1 << 2)
#define YG_CHANGE_GEOMETRY  (1 << 3)
#define YG_CHANGE_TEXTURE   (1 << 4)

class pfGroup;

class ygNode
    {
    public:
     ygNode(const char* name,bool master=true,bool createPfNode=true);
     virtual ~ygNode(void);
     virtual void reset(void);
     virtual void message(const ygMessage&);
     virtual void app(void);
     virtual void appTraverse(void);
     virtual void updateDebug(void);
          void resetTree(void);
         const ygString& name(void) const;
          ygNode* parent(void) const;
          int numChildren(void) const;
          ygNode* child(int) const;
          void addChild(ygNode*,bool doLocal=false);
          void removeChild(ygNode*,bool doLocal=false);
          void when(const ygString&,const ygString& message);
         void eventOccurred(const ygString&,const ygString& args="");
         pfGroup* pfnode(void) const;
         pfVec3 origin(const ygNode *other=NULL) const;
         void getTransform(pfMatrix& mat) const;
         void getParentTransform(pfMatrix& mat) const;
     virtual void addInitMessage(const ygString&);
         void sendNetMessage(const ygString&);
         void sendNetMessage(const ygMessage&);
         void receiveNetMessage(const ygString&,void *data);
     virtual void requestNetMessages(void) const;
     virtual void sendMessage(const ygMessage&,bool force=false);
         void parseNetMessage(const ygMessage&,bool force=false);
         void netMessageChanged(const ygString&,bool force=false);
         bool isNetMaster(void) const;
         void setChanged(uint);
         bool isChanged(uint);
         const ygString& netClassName(void) const;
         const ygString& className(void) const;
         const ygString& classDerivation(void) const;
         ygString* getNetMessageArg(const ygString&);
         const std::vector<ygMessage*>& getInitMessages(void);
         const std::vector<ygMessage*>& getNetMessages(bool all=true);
         bool isOfClass(const ygString&) const;
         void printMessages(std::ostream&,int indent=0);
         void printTimingStats(int indent=0);
    protected:
         void setClassName(const ygString&,bool net=false);
         void debugFlag(const ygString&,bool *);
         void addNetMessage(const ygString&, void*, int);
         void addNetMessageMap(const ygString&, const ygString&, int);
         void resetNetMessage(const ygString&);
          void clearEventMessages(void);
         void clearInitMessages(void);
         void setPfNode(pfGroup*);
         void timingBegin(void);
         void timingAppDone(void);
         void timingEnd(void);
         void dontSendNetMessage(const ygString&);
         void doSendNetMessage(const ygString&);
         void dontSaveNetMessage(const ygString&);
         void unreliableMessage(const ygString&);
         void reliableMessage(const ygString&);
    private:
         void sendInitMessages(void);
         void updateChildrenMessage(void);
         void removeAllChildren(void);
         void storeChildrenMessage(void);
     struct _ygNodePrivateData * p_;
    };

extern "C" ygNode* construct_ygNode(const char*,bool);

#endif
