// Revision: 11/01/01 Dave Pape

#include <map>
#include "ygUtil.h"
#include "ygTokenStream.h"
#include "ygNode.h"
#include "ygTransform.h"
#include "ygGeometry.h"
#include "ygObject.h"
#include "ygTexture.h"
#include "ygLight.h"
#include "ygSwitch.h"
#include "ygSelector.h"
#include "ygUser.h"
#include "ygWand.h"
#include "ygSound.h"
#include "ygSpace.h"
#include "ygEnvironment.h"
#include "ygCAVEHead.h"
#include "ygCAVENavigator.h"
#include "ygCAVETracker.h"
#include "ygCAVEWand.h"
#include "ygDSOLoader.h"
#include "ygNet.h"
#include "ygNodeDB.h"

using namespace std;


struct _ygNodeDBPrivateData
    {
    map<ygString,ygNode*> nodes;
    map<ygString, vector<ygNode*> > nodesByType;
    };

struct _ygNodeDBPrivateData* ygNodeDB::p_ = NULL;



void ygNodeDB::init(void)
    {
    if (!p_)
        p_ = new struct _ygNodeDBPrivateData;
    }


ygNode* ygNodeDB::find(const ygString& name,bool connect)
    {
    ygNode* node = p_->nodes[name];
    if ((!node) && (connect))
        node = ygNet::connectNewNode(name);
    return node;
    }


bool ygNodeDB::createValidName(const ygString& nodeType,ygString& name)
    {
    ygNode* node = p_->nodes[name];
    if (node)
        {
        while(node)
            {
            ygIncrementName(name);
            node = p_->nodes[name];
            }
        return false;
        }
    else if (name.length() == 0)
        {
        vector<ygNode*> nodes = p_->nodesByType[nodeType];
        int num = nodes.size();
        do
            {
            name = nodeType;
            char index[32];
            num++;
            sprintf(index,"%d",num);
            name += index;
            }
        while(node = p_->nodes[name]);
        return true;
        }
    return true;
    }


ygNode * ygNodeDB::create(const ygString& nodeType,const ygString& nodeName,
              ygTokenStream *tStream)
    {
    if (p_->nodes[nodeName] != NULL)
        cerr << "ERROR: ygNodeDB::create: a node named '" << nodeName
            << "' already exists\n";
    ygNode *node = NULL;
    ygNodeConstructorPtr constructPtr = ygDSOLoader::getNodeConstructor(nodeType);
    if (constructPtr)
        node = (*constructPtr)(nodeName.c_str(),true);
    if (!node)
        {
// NOTE: This may cause problems on SGIs when using STL strings!!
        cerr << "ERROR: ygNodeDB::create: unknown node type '"
            << nodeType << "'";
        if (tStream)
            cerr << " (file " << tStream->filename() << ", line "
                << tStream->linenumber() << ")";
        cerr << " - replacing with Group" << endl;
        node = new ygNode(nodeName.c_str());
        }
    if (node)
        {
        p_->nodes[nodeName] = node;
        p_->nodesByType[node->className()].push_back(node);
        ygNet::storeNode(node);
        }
    return node;
    }


void ygNodeDB::remove(ygNode* node)
    {
    map<ygString,ygNode*>::iterator i;
    for (i=p_->nodes.begin(); i != p_->nodes.end(); ++i)
        {
        if ((*i).second == node)
            p_->nodes.erase(i);
        }
    ygString className = node->className();
    vector<ygNode*>::iterator j;
    for (j = p_->nodesByType[className].begin(); j != p_->nodesByType[className].end(); j++)
        {
        if ((*j) == node)
            {
            p_->nodesByType[className].erase(j);
            break;
            }
        }
    }        


ygNode * ygNodeDB::createProxy(const ygString& proxyType,const ygString& nodeName)
    {
    ygNode *node = NULL;
    ygNodeConstructorPtr constructPtr = ygDSOLoader::getNodeConstructor(proxyType);
    if (constructPtr)
        node = (*constructPtr)(nodeName.c_str(),false);
    if (!node)
        {
        cerr << "ERROR: ygNodeDB::createProxy: unknown node type '"
            << proxyType << "' - replacing with Group" << endl;
        node = new ygNode(nodeName.c_str(),false);
        }
    if (node)
        {
        p_->nodes[nodeName] = node;
        p_->nodesByType[node->className()].push_back(node);
        node->requestNetMessages();
        }
    return node;
    }


const vector<ygNode*>& ygNodeDB::nodesOfType(const ygString& nodeType)
    {
    return p_->nodesByType[nodeType];
    }
