// Description: a node that loads a scene file
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> note: this node does not recreate scene files upon reset
//</ul>
//
// Category: Geometry
// Author: Alex Hill
// Revision: 09/01/04
//
#include <ygWorld.h>
#include <ygNetMessages.h>
#include <ygParser.h>
#include <ygMessage.h>
#include "ygScene.h"

using namespace std;

extern "C" ygNode* construct_ygScene(const char* name,bool master) { return new ygScene(name,master); }


ygScene::ygScene(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("ygScene",true);
    addNetMessage("file",&filename,YG_NET_STRING);
    loaded = false;
    first = true;
    }


ygScene::~ygScene(void)
    {
    }


void ygScene::reset(void)
    {
    clearEventMessages();
    const vector<ygMessage*>& messages = getInitMessages();
    vector<ygMessage*>::const_iterator iter;
    for (iter=messages.begin(); iter != messages.end(); ++iter)
        {
        if (loaded || (*iter)->recipient() == this)
            ygWorld::world()->scheduleMessage(**iter,true);
        }
    }


void ygScene::message(const ygMessage& msg)
    {
    //name of scene file to load
    if (msg == "file")
        {
    loadFile(filename);
        }
    //delete scene related nodes and reload scene file
    else if (msg == "reload")
        {
        set<ygNode*> allChildren;
        for (int i=0;i<numChildren();i++)
            allChildren.insert(allChildren.begin(),child(i));
        set<ygNode*>::const_iterator iter;
        for (iter = sceneChildren.begin(); iter != sceneChildren.end();++iter)
            {
            if (allChildren.find(*iter) != allChildren.end())
                ygWorld::world()->deleteTree(*iter);
            }
        sceneChildren.clear();
        parser.clear();
        loaded = false;
        }
    else
        ygNode::message(msg);
    }


void ygScene::loadFile(const ygString& file)
    {
    if (isNetMaster() && !loaded && filename.length() > 0)
        {
        if (first)
            {
            const vector<ygMessage*>& messages = getInitMessages();
            vector<ygMessage*>::const_iterator iter;
            for (iter=messages.begin(); iter != messages.end(); ++iter)
                {
                ygString msg; 
                (*iter)->writeString(msg);
                initMessages.push_back(msg);
                }
            first = false;
            }
        else
            {
            clearInitMessages();
            for (int i=0;i<initMessages.size();i++)
                addInitMessage(initMessages[i]);
            }
        int firstChild = numChildren();
        parser.parseFile(filename,this);
        parser.replaceInitMessages(this);
        loaded = true;
        ygNode::reset();
        for (int i=firstChild;i<numChildren();i++)
            sceneChildren.insert(sceneChildren.begin(),child(i));
        }
    }


void ygScene::sendMessage(const ygMessage& msg,bool force)
    {
    ygMessage newMsg = msg;
    parser.replaceMessageName(&newMsg);
    parseNetMessage(newMsg,force);
    message(newMsg);
    }
