// Author: Dave Pape
//           11/01/01
// Revision: 06/28/05 Alex Hill - added depth option to ygPrintSceneGraph
//
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/time.h>
#include "ygNode.h"
#include "ygUtil.h"

using namespace std;

bool ygFindFileInYgPath(const ygString& filename, ygString& returnPath)
    {
    static ygString searchPath;
    if (searchPath.length() < 1)
        {
        if (getenv("YG_PATH"))
            searchPath = getenv("YG_PATH");
        else
            searchPath = ".";
        }
    return ygFindFile(filename,searchPath,returnPath);
    }


bool ygFindFile(const ygString& filename, const ygString& searchPath,
                   ygString& returnPath)
    {
    ygString testName(filename);
    ygString *dir;
    int i=0;
    struct stat statbuf;
    if (stat(testName.c_str(),&statbuf) == 0)
        {
        returnPath = testName;
        return true;
        }
    while (dir = searchPath.nextToken(":",&i))
        {
        testName = *dir;
        delete dir;
        testName += "/";
        testName += filename;
        if (stat(testName.c_str(),&statbuf) == 0)
            {
            returnPath = testName;
            return true;
            }
        }
    return false;
    }


static bool ygTimeInitted=false;
static struct timeval ygTimeStart;
static float ygTimeMicrosecondF=1.0f/1000000.0f;
static double ygTimeMicrosecondD=1.0/1000000.0;

float ygGetTime(void)
    {
    struct timeval t;
    if (!ygTimeInitted)
        {
        gettimeofday(&ygTimeStart,NULL);
        ygTimeInitted = true;
        }
    gettimeofday(&t,NULL);
    return (t.tv_sec - ygTimeStart.tv_sec) +
        (t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondF;
    }

double ygGetTimeD(void)
    {
    struct timeval t;
    if (!ygTimeInitted)
        {
        gettimeofday(&ygTimeStart,NULL);
        ygTimeInitted = true;
        }
    gettimeofday(&t,NULL);
    return (t.tv_sec - ygTimeStart.tv_sec) +
        (t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondD;
    }


static void printTree(ygNode* node,int depth,int max)
    {
    if (node && depth <= max)
        {
        int i;
        node->printMessages(cout,depth);
        for (i=0; i < node->numChildren(); i++)
            printTree(node->child(i),depth+1,max);
        }
    }


void ygPrintSceneGraph(ygNode* node,int max)
    {
    cout << "Scene Graph:\n";
    if (max < 0)
        {
        while(node->parent() && max < 0)
           {
           node = node->parent();
           max++;
           }
        max = 99;
        }
    printTree(node,1,max);
    }


void ygMakeValidNodeName(ygString& name)
    {
    char * s = name.c_str();
    int i;
    for (i=0; i < name.length(); i++)
        if (!isalnum(s[i]))
            s[i] = '_';
    }


void ygIncrementName(ygString& name)
    {
    char* s = name.c_str();
    int i;
    for (i=name.length()-1; i>0; i--)
        {
        if (!isdigit(s[i]))
            break;
        }
    ygString number;
    number.assign(name,i+1,name.length()-i-1);
    name.assign(name,0,i+1);
    int num = atoi(number.c_str());
    num++;
    char index[32];
    sprintf(index,"%d",num);
    name += index;
    }

#ifdef __sgi

bool ygCreateThread(ygThreadFuncPtr func,void *arg)
    {
    pid_t p = sproc((void(*)(void*))func,PR_SADDR,arg);
    if (p <= 0)
        {
        perror("ygCreateThread: sproc");
        return false;
        }
    return true;
    }

#else

#include <QUANTA/QUANTA.hxx>

bool ygCreateThread(ygThreadFuncPtr func,void *arg)
    {
    QUANTAts_thread_c *thread = new QUANTAts_thread_c;
    int retval = thread->create(func,arg);
    return (retval == 0);
    }

#endif
