#ifndef _ygWorld_h_
#define _ygWorld_h_

#include "ygNode.h"
class ygUser;


#define YG_ISECT_ALLMASK    (1 << 0)
#define YG_ISECT_WALLMASK    (1 << 1)
#define YG_ISECT_FLOORMASK    (1 << 2)
#define YG_ISECT_GRABMASK    (1 << 3)
#define YG_ISECT_TRIGGERMASK    (1 << 4)



class ygWorld
    {
    public:
     ygWorld(void);
     ~ygWorld(void);
     void frame(void);
     void reset(void);
     void scheduleMessage(ygMessage&,bool queue=false);
     void message(const ygMessage&);
     void createRoot(void);
     void setRoot(ygNode*);
     ygNode * root(void) const { return root_; }
     ygNode * localRoot(void) const { return localRoot_; }
     class pfScene* pfscene(void) const;
     void setClip(float near,float far);
     void setSkyColor(const pfVec3&);
     void fogOff(void);
     void setFog(int fogtype,const pfVec3& color,float onset,float opaque);
     void addUser(ygUser*);
     void removeUser(ygUser*);
     int numUsers(void) const;
     ygUser* user(int) const;
     ygNode* viewer(void) const;
     void deleteLocalScene(void);
     void deleteTree(ygNode*);
     void saveLocalScene(const ygString&);
     void saveToFile(const ygString&,ygNode*);
     void loadFromFile(const ygString&,ygNode*);
     void saveTree(std::ostream&,ygNode *,int);
     bool quit(void);
     static inline double frameTime(void) { return frameTime_; }
     static inline double frameDeltaTime(void) { return frameDeltaTime_; }
     static inline int frameNumber(void) { return frameNumber_; }
     static inline ygWorld* world(void) { return world_; }
    private:
     void sendMessage(ygMessage&);
     void sendClassMessage(const ygMessage&,const ygString&,ygNode *);
     void unlinkTree(ygNode*);
     ygNode *root_, *localRoot_;
     struct _ygWorldPrivateData * p_;
     static float frameTime_;
     static float frameDeltaTime_;
     static int frameNumber_;
     static ygWorld* world_;
    };


#endif
